/*
 * FILE --- tanker_sim.cc
 *
 * PURPOSE --- To provide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
 * n-input m-output Multilayer Perceptron (MLP) neural network (NN).
 *
 * Implements the member functions of a class for performing a simulation of
 * a simple cargo ship whose steering is controlled by a MLP controller.
 *
 */

// Include Files
#include "tanker_sim.hh"

TANKER_SIM::TANKER_SIM (double _sampling_period, int _simulation_per_cycle,
			double _initial_time, double *_initial_state) 
  : RK4_SIM(3, 1, (_sampling_period/((double)_simulation_per_cycle)), 
	    _initial_time, _initial_state)
{
  // samp_period >= integration_step
  
  // Initialize parameters
  sampling_period = _sampling_period;
  simulation_per_cycle = _simulation_per_cycle;

  lenght = 350.0;			
  speed = 5.0; 
  //  speed = 3.0; // Lower speed
  abar = 1.0;      
  bbar = 1.0;

  // The parameters for the tanker under "ballast" conditions 
  // (a heavy ship) are:
  K_0 = 5.88;
  tau_10 = -16.91;
  tau_20 = 0.45;
  tau_30 = 1.43;

  // The parameters for the tanker under "full" conditions (a ship
  // that weighs less than one under "ballast" conditions) are:
  // K_0=0.83;
  // tau_10=-2.88;
  // tau_20=0.38;
  // tau_30=1.07;

  K = K_0*(speed/lenght);
  tau_1 = tau_10*(lenght/speed);
  tau_2 = tau_20*(lenght/speed);
  tau_3 = tau_30*(lenght/speed);

}

TANKER_SIM::~TANKER_SIM()
{

}

inline void TANKER_SIM::DERIV(const double t, const double *x, 
			      const double *u, double *xdot)
{
  // This is where the derivative is defined: xdot = f(t,x,u)

  // Tanker Ship Dynamics
  xdot[0] = x[1];
  xdot[1] = x[2] + (K*tau_3/(tau_1*tau_2))*u[0];
  xdot[2] = - (1.0/tau_1 + 1.0/tau_2)*(x[2] + (K*tau_3/(tau_1*tau_2))*u[0])
            - (1.0/(tau_1*tau_2))*(abar*x[1]*x[1]*x[1] + bbar*x[1])
            + (K/(tau_1*tau_2))*u[0];
}

// Each sampling time based on the current control input 
// the tanker ship dynamics are simulated
void TANKER_SIM::simulate(double _input)
{
  // Assign the control input 
  ctrl_input[0] = _input;
  
  // Correct the initial condition of the plant
  //  if (current_time == 0.0)
  //    plant_state[2] = - (K*tau_3/(tau_1*tau_2))*_input;
  
  // Simulate one sampling period
  for (int ii=0; ii < simulation_per_cycle; ii++)
    {
      solve();
    }
}


